#!/usr/bin/env python
from ansible.module_utils.hashivault import hashivault_argspec
from ansible.module_utils.hashivault import hashivault_client
from ansible.module_utils.hashivault import hashivault_init
from ansible.module_utils.hashivault import hashiwrapper

ANSIBLE_METADATA = {'status': ['stableinterface'], 'supported_by': 'community', 'version': '1.1'}
DOCUMENTATION = '''
---
module: hashivault_unseal
version_added: "1.2.0"
short_description: Hashicorp Vault unseal module
description:
    - Module to unseal Hashicorp Vault.
options:
    keys:
        description:
            - vault key shard(s).
extends_documentation_fragment: hashivault
'''
EXAMPLES = '''
---
- hosts: localhost
  tasks:
    - hashivault_unseal:
      keys: '{{vault_keys}}'
'''


def main():
    argspec = hashivault_argspec()
    argspec['keys'] = dict(required=True, type='str', no_log=True)
    module = hashivault_init(argspec)
    result = hashivault_unseal(module.params)
    if result.get('failed'):
        module.fail_json(**result)
    else:
        module.exit_json(**result)


@hashiwrapper
def hashivault_unseal(params):
    keys = params.get('keys')
    client = hashivault_client(params)
    if client.sys.is_sealed():
        return {'status': client.sys.submit_unseal_keys(keys.split()), 'changed': True}
    else:
        return {'changed': False}


if __name__ == '__main__':
    main()



#!/usr/bin/env python

from ansible.module_utils.hashivault import hashivault_argspec
from ansible.module_utils.hashivault import hashivault_auth_client
from ansible.module_utils.hashivault import hashivault_init
from ansible.module_utils.hashivault import hashiwrapper

ANSIBLE_METADATA = {'status': ['stableinterface'], 'supported_by': 'community', 'version': '1.1'}
DOCUMENTATION = '''
---
module: hashivault_token_lookup
version_added: "3.3.0"
short_description: Hashicorp Vault token lookup module
description:
    - Module to look up / check for existence of tokens in Hashicorp Vault.
options:
    lookup_token:
        description:
            - token to lookup if different from auth token
        default: to authentication token
    accessor:
        description:
            - If set, lookups will use the this accessor token
    wrap_ttl:
        description:
            - Indicates that the response should be wrapped in a cubbyhole token with the requested TTL.
extends_documentation_fragment: hashivault
'''
EXAMPLES = '''
---
- hosts: localhost
  tasks:
    - name: "Lookup token"
      hashivault_token_lookup:
        lookup_token: "{{client_token}}"
      register: "vault_token_lookup"
'''


def main():
    argspec = hashivault_argspec()
    argspec['lookup_token'] = dict(required=False, type='str', no_log=True)
    argspec['accessor'] = dict(required=False, type='bool', default=False)
    argspec['wrap_ttl'] = dict(required=False, type='int')
    module = hashivault_init(argspec)
    result = hashivault_token_lookup(module.params)
    if result.get('failed'):
        module.fail_json(**result)
    else:
        module.exit_json(**result)


@hashiwrapper
def hashivault_token_lookup(params):
    client = hashivault_auth_client(params)
    accessor = params.get('accessor')
    lookup_token = params.get('lookup_token')
    if lookup_token is None:
        lookup_token = params.get('token')
    wrap_ttl = params.get('wrap_ttl')
    lookup = client.lookup_token(token=lookup_token, accessor=accessor, wrap_ttl=wrap_ttl)
    return {'changed': False, 'lookup': lookup}


if __name__ == '__main__':
    main()



#!/usr/bin/env python
from ansible.module_utils.hashivault import hashivault_argspec
from ansible.module_utils.hashivault import hashivault_auth_client
from ansible.module_utils.hashivault import hashivault_init
from ansible.module_utils.hashivault import hashiwrapper

ANSIBLE_METADATA = ANSIBLE_METADATA = {'status': ['stableinterface'], 'supported_by': 'community', 'version': '1.1'}
DOCUMENTATION = '''
---
module: hashivault_policy
version_added: "2.1.0"
short_description: Hashicorp Vault policy set module
description:
    - Module to set a policy in Hashicorp Vault. Use hashivault_policy instead.
options:
    name:
        description:
            - policy name.
    state:
        type: str
        choices: ["present", "absent"]
        default: present
        description:
            - present or absent
    rules:
        description:
            - policy rules.
    rules_file:
        description:
            - name of local file to read for policy rules.
extends_documentation_fragment: hashivault
'''
EXAMPLES = '''
---
- hosts: localhost
  tasks:
    - hashivault_policy:
        name: my_policy
        rules: '{{rules}}'
'''


def main():
    argspec = hashivault_argspec()
    argspec['name'] = dict(required=True, type='str')
    argspec['rules'] = dict(required=False, type='str')
    argspec['rules_file'] = dict(required=False, type='str')
    argspec['state'] = dict(required=False, choices=['present', 'absent'], default='present')
    mutually_exclusive = [['rules', 'rules_file']]
    module = hashivault_init(argspec, mutually_exclusive=mutually_exclusive, supports_check_mode=True)
    result = hashivault_policy(module)
    if result.get('failed'):
        module.fail_json(**result)
    else:
        module.exit_json(**result)


@hashiwrapper
def hashivault_policy(module):
    params = module.params
    client = hashivault_auth_client(params)
    state = params.get('state')
    name = params.get('name')
    exists = False
    changed = False
    current_state = {}
    desired_state = {}

    # get current policies
    current_policies = client.sys.list_policies()
    if isinstance(current_policies, dict):
        current_policies = current_policies.get('policies', current_policies)
    if name in current_policies:
        exists = True
        current_state = client.get_policy(name)

    # Define desired rules
    rules_file = params.get('rules_file')
    if rules_file:
        try:
            desired_state = open(rules_file, 'r').read()
        except Exception as e:
            return {'changed': False,
                    'failed': True,
                    'msg': 'Error opening rules file <%s>: %s' % (rules_file, str(e))}
    else:
        desired_state = params.get('rules')

    # Check required actions
    if state == 'present' and not exists:
        changed = True
    elif state == 'absent' and exists:
        changed = True
    elif state == 'present' and exists:
        if current_state != desired_state:
            changed = True

    if changed and not module.check_mode:
        # create or update
        if state == 'present':
            client.sys.create_or_update_policy(name, desired_state)
        # delete
        elif state == 'absent':
            client.sys.delete_policy(name)

    return {
        "changed": changed,
        "diff": {
            "before": current_state,
            "after": desired_state,
        },
    }


if __name__ == '__main__':
    main()



import os

import hvac
import requests
from ansible.module_utils.basic import AnsibleModule, env_fallback
from hvac.exceptions import InvalidPath

normalize = {'list': list, 'str': str, 'dict': dict, 'bool': bool, 'int': int, 'duration': str}


def hashivault_argspec():
    argument_spec = dict(
        url=dict(required=False, default=os.environ.get('VAULT_ADDR', ''), type='str'),
        ca_cert=dict(required=False, default=os.environ.get('VAULT_CACERT', ''), type='str'),
        ca_path=dict(required=False, default=os.environ.get('VAULT_CAPATH', ''), type='str'),
        client_cert=dict(required=False, default=os.environ.get('VAULT_CLIENT_CERT', ''), type='str'),
        client_key=dict(required=False, default=os.environ.get('VAULT_CLIENT_KEY', ''), type='str'),
        verify=dict(required=False, default=(not os.environ.get('VAULT_SKIP_VERIFY', '')), type='bool'),
        authtype=dict(required=False, default=os.environ.get('VAULT_AUTHTYPE', 'token'), type='str'),
        login_mount_point=dict(required=False, default=os.environ.get('VAULT_LOGIN_MOUNT_POINT', None), type='str'),
        token=dict(required=False, fallback=(hashivault_default_token, ''), type='str', no_log=True),
        username=dict(required=False, default=os.environ.get('VAULT_USER', ''), type='str'),
        password=dict(required=False, fallback=(env_fallback, ['VAULT_PASSWORD']), type='str', no_log=True),
        role_id=dict(required=False, fallback=(env_fallback, ['VAULT_ROLE_ID']), type='str', no_log=True),
        secret_id=dict(required=False, fallback=(env_fallback, ['VAULT_SECRET_ID']), type='str', no_log=True),
        aws_header=dict(required=False, fallback=(env_fallback, ['VAULT_AWS_HEADER']), type='str', no_log=True),
        namespace=dict(required=False, default=os.environ.get('VAULT_NAMESPACE', None), type='str'),
        timeout=dict(required=False, default=30, type=int)
    )
    return argument_spec


def hashivault_init(argument_spec, supports_check_mode=False, required_if=None, required_together=None,
                    required_one_of=None, mutually_exclusive=None):
    module = AnsibleModule(argument_spec=argument_spec, supports_check_mode=supports_check_mode,
                           required_if=required_if, required_together=required_together,
                           required_one_of=required_one_of, mutually_exclusive=mutually_exclusive)
    module.no_log_values.discard('0')
    module.no_log_values.discard(0)
    module.no_log_values.discard('1')
    module.no_log_values.discard(1)
    module.no_log_values.discard(True)
    module.no_log_values.discard(False)
    module.no_log_values.discard('ttl')
    return module


def hashivault_normalize_from_doc(module, options, documentation):
    desired_state = {}
    for key, value in options.items():
        config_type = documentation.get(key, {}).get('type')
        if config_type is None:
            module.warn('Unknown option "{}". Make sure this is not a typo, if it is not, please open an '
                        'issue at https://github.com/TerryHowe/ansible-modules-hashivault/issues.'.format(key))
        elif value is not None:
            try:
                value = normalize[config_type](value)
            except Exception as e:
                raise Exception({
                    'changed': False,
                    'failed': True,
                    'msg':
                        'config item \'{}\' with value \'{}\' could not be converted to \'{}\': {}'
                        .format(key, value, config_type, "\n".join(e.args))})

        desired_state[key] = value

    return desired_state


def get_ec2_iam_role():
    request = requests.get(url='http://169.254.169.254/latest/meta-data/iam/security-credentials/')
    request.raise_for_status()
    return request.content


def get_ec2_iam_credentials(header_value, role_id):
    role_name = get_ec2_iam_role()
    metadata_url = 'http://169.254.169.254/latest/meta-data/iam/security-credentials/{role}'.format(
        role=role_name
    )
    response = requests.get(url=metadata_url)
    response.raise_for_status()
    credentials = response.json()
    return dict(
        access_key=credentials['AccessKeyId'],
        secret_key=credentials['SecretAccessKey'],
        session_token=credentials['Token'],
        header_value=header_value,
        role=role_id
    )


def hashivault_client(params):
    url = params.get('url')
    ca_cert = params.get('ca_cert')
    ca_path = params.get('ca_path')
    client_cert = params.get('client_cert')
    client_key = params.get('client_key')
    cert = (client_cert, client_key)
    check_verify = params.get('verify')
    namespace = params.get('namespace', None)
    timeout = params.get('timeout')
    if check_verify == '' or check_verify:
        if ca_cert:
            verify = ca_cert
        elif ca_path:
            verify = ca_path
        else:
            verify = check_verify
    else:
        verify = check_verify
    client = hvac.Client(url=url, cert=cert, verify=verify, namespace=namespace, timeout=timeout)
    return client


def hashivault_auth(client, params):
    token = params.get('token')
    authtype = params.get('authtype')
    login_mount_point = params.get('login_mount_point', authtype)
    if not login_mount_point:
        login_mount_point = authtype
    username = params.get('username')
    password = params.get('password')
    secret_id = params.get('secret_id')
    role_id = params.get('role_id')

    if authtype == 'github':
        client.auth.github.login(token, mount_point=login_mount_point)
    elif authtype == 'userpass':
        client.auth.userpass.login(username, password, mount_point=login_mount_point)
    elif authtype == 'ldap':
        client.auth.ldap.login(username, password, mount_point=login_mount_point)
    elif authtype == 'approle':
        client = AppRoleClient(client, role_id, secret_id, mount_point=login_mount_point)
    elif authtype == 'tls':
        client.auth.cert.login()
    elif authtype == 'aws':
        credentials = get_ec2_iam_credentials(params.get['aws_header'], role_id)
        client.auth_aws_iam(**credentials)
    else:
        client.token = token
    return client


def hashivault_auth_client(params):
    client = hashivault_client(params)
    return hashivault_auth(client, params)


def hashiwrapper(function):
    def wrapper(*args, **kwargs):
        result = {"changed": False, "rc": 0}
        result.update(function(*args, **kwargs))
        return result
    return wrapper


def hashivault_default_token():
    """Get a default Vault token from an environment variable or a file."""
    if 'VAULT_TOKEN' in os.environ:
        return os.environ['VAULT_TOKEN']
    token_file = os.path.expanduser('~/.vault-token')
    if os.path.exists(token_file):
        with open(token_file, 'r') as f:
            return f.read().strip()
    return ''


@hashiwrapper
def hashivault_read(params):
    result = {"changed": False, "rc": 0}
    client = hashivault_auth_client(params)
    version = params.get('version')
    mount_point = params.get('mount_point')
    secret = params.get('secret')
    secret_version = params.get('secret_version')

    key = params.get('key')
    default = params.get('default')
    if secret.startswith('/'):
        secret = secret.lstrip('/')
        mount_point = ''
    if mount_point:
        secret_path = '%s/%s' % (mount_point, secret)
    else:
        secret_path = secret

    try:
        if version == 2:
            response = client.secrets.kv.v2.read_secret_version(secret, mount_point=mount_point, version=secret_version)
        else:
            response = client.secrets.kv.v1.read_secret(secret, mount_point=mount_point)
    except InvalidPath:
        response = None
    except Exception as e:
        result['rc'] = 1
        result['failed'] = True
        error_string = "%s(%s)" % (e.__class__.__name__, e)
        result['msg'] = u"Error %s reading %s" % (error_string, secret_path)
        return result
    if not response:
        if default is not None:
            result['value'] = default
            return result
        result['rc'] = 1
        result['failed'] = True
        result['msg'] = u"Secret %s is not in vault" % secret_path
        return result
    if version == 2:
        try:
            data = response.get('data', {})
            metadata = data.get('metadata', {})
            data = data.get('data', {})
        except Exception:
            data = str(response)
    else:
        data = response['data']
        metadata = {}
    lease_duration = response.get('lease_duration', None)
    if lease_duration is not None:
        result['lease_duration'] = lease_duration
    lease_id = response.get('lease_id', None)
    if lease_id is not None:
        result['lease_id'] = lease_id
    renewable = response.get('renewable', None)
    if renewable is not None:
        result['renewable'] = renewable
    wrap_info = response.get('wrap_info', None)
    if wrap_info is not None:
        result['wrap_info'] = wrap_info
    if key and key not in data:
        if default is not None:
            result['value'] = default
            return result
        result['rc'] = 1
        result['failed'] = True
        result['msg'] = u"Key %s is not in secret %s" % (key, secret_path)
        return result
    if key:
        value = data[key]
    else:
        value = data
    result['value'] = value
    result['metadata'] = metadata
    return result


class AppRoleClient(object):
    """
    hvac.Client decorator which generates and sets a new approle token on every
    function call. This allows multiple calls to Vault without having to manually
    generate and set a token on every Vault call.
    """

    def __init__(self, client, role_id, secret_id, mount_point):
        object.__setattr__(self, 'client', client)
        object.__setattr__(self, 'role_id', role_id)
        object.__setattr__(self, 'secret_id', secret_id)
        object.__setattr__(self, 'login_mount_point', mount_point)

    def __setattr__(self, name, val):
        """
        sets attribute in decorated class (Client)
        """
        client = object.__getattribute__(self, 'client')
        client.__setattr__(name, val)

    def __getattribute__(self, name):
        """
        generates and sets new approle token in decorated class (Client)
        returns decorated class (Client) attribute
        """
        client = object.__getattribute__(self, 'client')
        attr = client.__getattribute__(name)

        role_id = object.__getattribute__(self, 'role_id')
        secret_id = object.__getattribute__(self, 'secret_id')
        login_mount_point = object.__getattribute__(self, 'login_mount_point')
        resp = client.auth.approle.login(role_id, secret_id=secret_id, mount_point=login_mount_point)
        client.token = str(resp['auth']['client_token'])
        return attr


def _compare_state(desired_state, current_state, ignore=None):
    """Compares desired state to current state. Returns true if objects are equal

    Recursively walks dict object to compare all keys

    :param desired_state: The state user desires.
    :param current_state: The state that currently exists.
    :param ignore: Ignore these keys.
    :type ignore: list

    :return: True if the states are the same.
    :rtype: bool
    """

    if ignore is None:
        ignore = []
    if (type(desired_state) is list):
        if ((type(current_state) != list) or (len(desired_state) != len(current_state))):
            return False
        for i in range(len(desired_state)):
            if (not _compare_state(desired_state[i], current_state[i])):
                return False
        return True

    if (type(desired_state) is dict):
        if (type(current_state) != dict):
            return False

        # iterate over dictionary keys
        for key in desired_state.keys():
            if key in ignore:
                continue
            v = desired_state[key]
            if ((key not in current_state) or (not _compare_state(v, current_state.get(key)))):
                return False
        return True

    # lots of things get handled as strings in ansible that aren't necessarily strings, can extend this list later.
    if isinstance(desired_state, str) and isinstance(current_state, int):
        current_state = str(current_state)

    return ((desired_state == current_state))


def _convert_to_seconds(original_value):
    try:
        value = str(original_value)
        seconds = 0
        if 'h' in value:
            ray = value.split('h')
            seconds = int(ray.pop(0)) * 3600
            value = ''.join(ray)
        if 'm' in value:
            ray = value.split('m')
            seconds += int(ray.pop(0)) * 60
            value = ''.join(ray)
        if value:
            ray = value.split('s')
            seconds += int(ray.pop(0))
        return seconds
    except Exception:
        pass
    return original_value


def get_keys_updated(desired_state, current_state, ignore=None):
    """Return list of keys that have different values

    Recursively walks dict object to compare all keys

    :param desired_state: The state user desires.
    :type desired_state: dict
    :param current_state: The state that currently exists.
    :type current_state: dict
    :param ignore: Ignore these keys.
    :type ignore: list

    :return: Different items
    :rtype: list
    """

    if ignore is None:
        ignore = []

    differences = []
    for key in desired_state.keys():
        if key in ignore:
            continue
        if (key not in current_state):
            differences.append(key)
            continue
        new_value = desired_state[key]
        old_value = current_state[key]
        if 'ttl' in key:
            if _convert_to_seconds(old_value) != _convert_to_seconds(new_value):
                differences.append(key)
        elif not _compare_state(new_value, old_value):
            differences.append(key)
    return differences


def is_state_changed(desired_state, current_state, ignore=None):
    """Return list of keys that have different values

    Recursively walks dict object to compare all keys

    :param desired_state: The state user desires.
    :type desired_state: dict
    :param current_state: The state that currently exists.
    :type current_state: dict
    :param ignore: Ignore these keys.
    :type ignore: list

    :return: Different
    :rtype: bool
    """
    return(len(get_keys_updated(desired_state, current_state)) > 0)



#!/usr/bin/env python
from ansible.module_utils.hashivault import hashivault_auth_client
from ansible.module_utils.hashivault import hashivault_argspec
from ansible.module_utils.hashivault import hashivault_init
from ansible.module_utils.hashivault import hashiwrapper

ANSIBLE_METADATA = {'status': ['preview'], 'supported_by': 'community', 'version': '1.1'}
DOCUMENTATION = r'''
---
module: hashivault_pki_cert_revoke
version_added: "4.5.0"
short_description: Hashicorp Vault PKI Revoke Certificate
description:
    - This module revokes a certificate using its serial number.
    - This is an alternative option to the standard method of revoking using Vault lease IDs.
    - A successful revocation will rotate the CRL.
options:
    serial:
        recuired: true
        description:
            - Specifies the serial number of the certificate to revoke, in hyphen-separated or colon-separated octal.
    mount_point:
        default: pki
        description:
        - location where secrets engine is mounted. also known as path
extends_documentation_fragment:
    - hashivault
'''

EXAMPLES = r'''
---
- hosts: localhost
  tasks:
    - hashivault_pki_cert_revoke:
        role: 'tester'
        common_name: 'test.example.com'
      register: cert
    - debug: msg="{{ cert }}"

'''


def main():
    argspec = hashivault_argspec()
    argspec['serial'] = dict(required=True, type='str')
    argspec['mount_point'] = dict(required=False, type='str', default='pki')

    module = hashivault_init(argspec)
    result = hashivault_pki_cert_revoke(module)

    if result.get('failed'):
        module.fail_json(**result)
    else:
        module.exit_json(**result)


@hashiwrapper
def hashivault_pki_cert_revoke(module):
    params = module.params
    client = hashivault_auth_client(params)

    serial = params.get('serial')
    mount_point = params.get('mount_point').strip('/')

    result = {"changed": False, "rc": 0}
    try:
        result['data'] = client.secrets.pki.revoke_certificate(serial_number=serial,
                                                               mount_point=mount_point).get('data')
    except Exception as e:
        result['rc'] = 1
        result['failed'] = True
        result['msg'] = u"Exception: " + str(e)
    return result


if __name__ == '__main__':
    main()



#!/usr/bin/env python
from ansible.module_utils.hashivault import hashivault_argspec
from ansible.module_utils.hashivault import hashivault_client
from ansible.module_utils.hashivault import hashivault_init
from ansible.module_utils.hashivault import hashiwrapper

ANSIBLE_METADATA = {'status': ['stableinterface'], 'supported_by': 'community', 'version': '1.1'}
DOCUMENTATION = '''
---
module: hashivault_rekey_init
version_added: "3.3.0"
short_description: Hashicorp Vault rekey init module
description:
    - Module to start rekey Hashicorp Vault.
options:
    secret_shares:
        description:
            - specifies the number of shares to split the master key into.
        default: 5
    secret_threshold:
        description:
            - specifies the number of shares required to reconstruct the master key.
        default: 3
    pgp_keys:
        description:
            - specifies an array of PGP public keys used to encrypt the output unseal keys.
        default: []
    backup:
        description:
            - specifies if using PGP-encrypted keys, whether Vault should also store a plaintext backup of the
              PGP-encrypted keys at core/unseal-keys-backup in the physical storage backend. These can then be
              retrieved and removed via the sys/rekey/backup endpoint.
        default: False
    verification_required:
        description:
            - when verification is turned on, after successful authorization with the current unseal keys, the new
              unseal keys are returned but the master key is not actually rotated. The new keys must be provided to
              authorize the actual rotation of the master key. This ensures that the new keys have been successfully
              saved and protects against a risk of the keys being lost after rotation but before they can be persisted.
        default: False
extends_documentation_fragment: hashivault
'''
EXAMPLES = '''
---
- hosts: localhost
  tasks:
    - hashivault_rekey_init:
        secret_shares: 7
        secret_threshold: 4
'''


def main():
    argspec = hashivault_argspec()
    argspec['secret_shares'] = dict(required=False, type='int', default=5)
    argspec['secret_threshold'] = dict(required=False, type='int', default=3)
    argspec['pgp_keys'] = dict(required=False, type='list', default=[], no_log=True)
    argspec['backup'] = dict(required=False, type='bool', default=False)
    argspec['verification_required'] = dict(required=False, type='bool', default=False)
    module = hashivault_init(argspec)
    result = hashivault_rekey_init(module.params)
    if result.get('failed'):
        module.fail_json(**result)
    else:
        module.exit_json(**result)


@hashiwrapper
def hashivault_rekey_init(params):
    client = hashivault_client(params)
    # Check if rekey is on-going, exit if there is a rekey in progress
    status = client.rekey_status
    if status['started']:
        return {'changed': False}
    secret_shares = params.get('secret_shares')
    secret_threshold = params.get('secret_threshold')
    pgp_keys = params.get('pgp_keys')
    backup = params.get('backup')
    verification_required = params.get('verification_required')
    status = client.sys.start_rekey(secret_shares=secret_shares, secret_threshold=secret_threshold,
                                    pgp_keys=pgp_keys, backup=backup,
                                    require_verification=verification_required)
    return {'status': status, 'changed': True}


if __name__ == '__main__':
    main()



#!/usr/bin/env python
from ansible.module_utils.hashivault import hashivault_argspec
from ansible.module_utils.hashivault import hashivault_client
from ansible.module_utils.hashivault import hashivault_init
from ansible.module_utils.hashivault import hashiwrapper

ANSIBLE_METADATA = {'status': ['stableinterface'], 'supported_by': 'community', 'version': '1.1'}
DOCUMENTATION = '''
---
module: hashivault_rekey_verify
version_added: "4.6.2"
short_description: Hashicorp Vault rekey verify module
description:
    - Module to verify a rekey of Hashicorp Vault. Requires that a rekey
      be started with hashivault_rekey_init passing verification_required == True
      and being successfully completed.
options:
    key:
        description:
            - vault key shard (aka unseal key). The new, freshly generated one.
    nonce:
        description:
            - verify nonce.
extends_documentation_fragment: hashivault
'''
EXAMPLES = '''
---
- hosts: localhost
  tasks:
    - hashivault_rekey_verify:
      key: '{{vault_key}}'
      nonce: '{{nonce}}'
'''


def main():
    argspec = hashivault_argspec()
    argspec['key'] = dict(required=True, type='str', no_log=True)
    argspec['nonce'] = dict(required=True, type='str')
    module = hashivault_init(argspec)
    result = hashivault_rekey_verify(module.params)
    if result.get('failed'):
        module.fail_json(**result)
    else:
        module.exit_json(**result)


@hashiwrapper
def hashivault_rekey_verify(params):
    key = params.get('key')
    nonce = params.get('nonce')
    client = hashivault_client(params)
    return {'status': client.sys.rekey_verify(key, nonce), 'changed': True}


if __name__ == '__main__':
    main()



#!/usr/bin/env python
from ansible.module_utils.hashivault import hashivault_argspec
from ansible.module_utils.hashivault import hashivault_auth_client
from ansible.module_utils.hashivault import hashivault_init
from ansible.module_utils.hashivault import hashiwrapper

ANSIBLE_METADATA = {'status': ['stableinterface'], 'supported_by': 'community', 'version': '1.1'}
DOCUMENTATION = '''
---
module: hashivault_oidc_auth_role
version_added: "4.1.1"
short_description: Hashicorp Vault OIDC secret engine role
description:
    - Module to define an OIDC role that vault can generate dynamic credentials for vault
options:
    mount_point:
        description:
            - name of the secret engine mount name.
        default: oidc
    name:
        description:
            - name of the role in vault
    bound_audiences:
        description:
            - List of `aud` claims to match against. Any match is sufficient.
    user_claim:
        description:
            - The claim to use to uniquely identify the user; this will be used as the name for the Identity entity
              alias created due to a successful login. The claim value must be a string.
        default: sub
    bound_subject:
        description:
            - If set, requires that the sub claim matches this value.
    bound_claims:
        description:
            - If set, a map of claims/values to match against. The expected value may be a single string or a list of
              strings.
    groups_claim:
        description:
            - The claim to use to uniquely identify the set of groups to which the user belongs; this will be used as
              the names for the Identity group aliases created due to a successful login. The claim value must be a
              list of strings.
    claim_mappings:
        description:
            - If set, a map of claims (keys) to be copied to specified metadata fields (values).
    oidc_scopes:
        description:
            - If set, a list of OIDC scopes to be used with an OIDC role. The standard scope "openid" is automatically
              included and need not be specified.
    allowed_redirect_uris:
        description:
            - The list of allowed values for redirect_uri during OIDC logins.
            - When using nested namespaces, use url encoding '%2F' instead of '/'
    token_ttl:
        description:
            - The incremental lifetime for generated tokens. This current value of this will be referenced at renewal
              time.
    token_max_ttl:
        description:
            - The maximum lifetime for generated tokens. This current value of this will be referenced at renewal time.
    token_policies:
        description:
            - List of policies to encode onto generated tokens. Depending on the auth method, this list may be
              supplemented by user/group/other values.
    token_bound_cidrs:
        description:
            - List of CIDR blocks; if set, specifies blocks of IP addresses which can authenticate successfully, and
              ties the resulting token to these blocks as well.
    token_explicit_max_ttl:
        description:
            - If set, will encode an explicit max TTL onto the token. This is a hard cap even if token_ttl and
              token_max_ttl would otherwise allow a renewal.
    token_no_default_policy:
        description:
            - If set, the default policy will not be set on generated tokens; otherwise it will be added to the policies
              set in token_policies.
    token_num_uses:
        description:
            - The maximum number of times a generated token may be used (within its lifetime); 0 means unlimited.
    token_period:
        description:
            - If set, indicates that the token generated using this role should never expire. The token should be
              renewed within the duration specified by this value. At each renewal, the token's TTL will be set to
              the value of this parameter.
    token_type:
        description:
            - The type of token that should be generated. Can be service, batch, or default to use the mount's tuned
              default (which unless changed will be service tokens). For token store roles, there are two additional
              possibilities (default-service and default-batch) which specify the type to return unless the client
              requests a different type at generation time.
extends_documentation_fragment: hashivault
'''
EXAMPLES = '''
---
- hosts: localhost
  tasks:
    - hashivault_oidc_auth_role:
        name: "gmail"
        bound_audiences: ["123-456.apps.googleusercontent.com"]
        allowed_redirect_uris: ["https://vault.com:8200/ui/vault/auth/oidc/oidc/callback"]
        token_policies: ["test"]

- hosts: localhost
  tasks:
    - hashivault_oidc_auth_role:
        name: nested_ns_role
        bound_audiences: ["123-456.apps.googleusercontent.com"]
        allowed_redirect_uris: ["https://vault.com:8200/ui/oidc/oidc/callback?namespace=namespaceone%2Fnamespacetwo"]
        token_policies: ["test"]
'''


def main():
    argspec = hashivault_argspec()
    argspec['state'] = dict(required=False, type='str', default='present', choices=['present', 'absent'])
    argspec['name'] = dict(required=True, type='str')
    argspec['mount_point'] = dict(required=False, type='str', default='oidc')
    argspec['user_claim'] = dict(required=False, type='str', default='sub')
    argspec['allowed_redirect_uris'] = dict(required=True, type='list')
    argspec['bound_audiences'] = dict(required=False, type='list', default=[])
    argspec['bound_subject'] = dict(required=False, type='str', default='')
    argspec['bound_claims'] = dict(required=False, type='dict')
    argspec['groups_claim'] = dict(required=False, type='str', default='')
    argspec['claim_mappings'] = dict(required=False, type='dict')
    argspec['oidc_scopes'] = dict(required=False, type='list', default=[])
    argspec['token_ttl'] = dict(required=False, type='int', default=0)
    argspec['token_max_ttl'] = dict(required=False, type='int', default=0)
    argspec['token_policies'] = dict(required=False, type='list', default=[])
    argspec['policies'] = dict(required=False, type='list', default=[])
    argspec['token_bound_cidrs'] = dict(required=False, type='list', default=[])
    argspec['token_explicit_max_ttl'] = dict(required=False, type='int', default=0)
    argspec['token_no_default_policy'] = dict(required=False, type='bool', default=False)
    argspec['token_num_uses'] = dict(required=False, type='int', default=0)
    argspec['token_period'] = dict(required=False, type='int', default=0)
    argspec['token_type'] = dict(required=False, type='str', default='default')
    argspec['clock_skew_leeway'] = dict(required=False, type='int', default=0)
    argspec['expiration_leeway'] = dict(required=False, type='int', default=0)
    argspec['not_before_leeway'] = dict(required=False, type='int', default=0)
    argspec['role_type'] = dict(required=False, type='str', default='oidc', choices=['oidc', 'jwt'])

    module = hashivault_init(argspec, supports_check_mode=True)
    result = hashivault_oidc_auth_role(module)
    if result.get('failed'):
        module.fail_json(**result)
    else:
        module.exit_json(**result)


@hashiwrapper
def hashivault_oidc_auth_role(module):
    params = module.params
    mount_point = params.get('mount_point').strip('/')
    name = params.get('name').strip('/')
    state = params.get('state')
    client = hashivault_auth_client(params)
    parameters = [
        'allowed_redirect_uris',
        'bound_audiences',
        'bound_claims',
        'bound_subject',
        'claim_mappings',
        'groups_claim',
        'oidc_scopes',
        'token_bound_cidrs',
        'token_explicit_max_ttl',
        'token_ttl',
        'token_max_ttl',
        'token_no_default_policy',
        'token_policies',
        'policies',
        'token_type',
        'user_claim',
        'token_period',
        'token_num_uses',
        'clock_skew_leeway',
        'expiration_leeway',
        'not_before_leeway',
        'role_type',
    ]
    desired_state = dict()
    for parameter in parameters:
        if params.get(parameter) is not None:
            desired_state[parameter] = params.get(parameter)
    desired_state['verbose_oidc_logging'] = False
    if not desired_state['token_policies'] and desired_state['policies']:
        desired_state['token_policies'] = desired_state['policies']
    desired_state.pop('policies', None)
    desired_state['path'] = mount_point

    changed = False
    current_state = {}
    try:
        current_state = client.auth.oidc.read_role(name=name, path=mount_point)['data']
    except Exception:
        changed = True
    for key in desired_state.keys():
        current_value = current_state.get(key, None)
        if current_value is not None and current_value != desired_state[key]:
            changed = True
            break

    if changed and not module.check_mode:
        if not current_state and state == 'present':
            client.auth.oidc.create_role(name=name, **desired_state)
        elif current_state and state == 'absent':
            client.auth.oidc.delete_role(name=name)

    return {
        'changed': changed,
        "diff": {
            "before": current_state,
            "after": desired_state,
        }
    }


if __name__ == '__main__':
    main()



#!/usr/bin/env python
from ansible.module_utils.hashivault import hashivault_argspec
from ansible.module_utils.hashivault import hashivault_auth_client
from ansible.module_utils.hashivault import hashivault_init
from ansible.module_utils.hashivault import hashiwrapper

ANSIBLE_METADATA = {'status': ['stableinterface'], 'supported_by': 'community', 'version': '1.1'}
DOCUMENTATION = '''
---
module: hashivault_oidc_auth_method_config
version_added: "4.1.1"
short_description: Hashicorp Vault OIDC auth method config
description:
    - Module to configure an OIDC auth mount
options:
    mount_point:
        description:
            - name of the secret engine mount name.
        default: oidc
    default_role:
        description:
            - The default role to use if none is provided during login.
    oidc_discovery_url:
        description:
            - The OIDC Discovery URL, without any .well-known component (base path). Cannot be used with "jwks_url" or
              "jwt_validation_pubkeys".
    oidc_client_id:
        description:
            - The OAuth Client ID from the provider for OIDC roles.
    oidc_client_secret:
        description:
            - The OAuth Client Secret from the provider for OIDC roles.
    bound_issuer:
        description:
            - The value against which to match the iss claim in a JWT.
    jwks_ca_pem:
        description:
            - The CA certificate or chain of certificates, in PEM format, to use to validate connections to the JWKS
              URL. If not set, system certificates are used.
    jwks_url:
        description:
            - JWKS URL to use to authenticate signatures. Cannot be used with "oidc_discovery_url" or
              "jwt_validation_pubkeys".
    jwt_supported_algs:
        description:
            - A list of supported signing algorithms.
        default: RS256
    jwt_validation_pubkeys:
        description:
            - A list of PEM-encoded public keys to use to authenticate signatures locally. Cannot be used with
              "jwks_url" or "oidc_discovery_url".
    oidc_discovery_ca_pem:
        description:
            - The CA certificate or chain of certificates, in PEM format, to use to validate connections to the OIDC
              Discovery URL. If not set, system certificates are used.
    provider_config:
        description:
            - "Configuration options for provider-specific handling.
              Providers with specific handling include: Azure, Google."
extends_documentation_fragment: hashivault
'''
EXAMPLES = '''
---
- hosts: localhost
  tasks:
    - hashivault_oidc_auth_method_config:
        oidc_discovery_url: "https://accounts.google.com"
        oidc_client_id: "123456"
        oidc_client_secret: "123456"
        default_role: "gmail"
'''


def main():
    argspec = hashivault_argspec()
    argspec['mount_point'] = dict(required=False, type='str', default='oidc')
    argspec['bound_issuer'] = dict(required=False, type='str', default='')
    argspec['jwks_ca_pem'] = dict(required=False, type='str', default='')
    argspec['jwks_url'] = dict(required=False, type='str')
    argspec['jwt_supported_algs'] = dict(required=False, type='list', default=[])
    argspec['jwt_validation_pubkeys'] = dict(required=False, type='list', default=[])
    argspec['oidc_discovery_url'] = dict(required=False, type='str')
    argspec['oidc_discovery_ca_pem'] = dict(required=False, type='str', default='')
    argspec['oidc_client_id'] = dict(required=False, type='str')
    argspec['oidc_client_secret'] = dict(required=False, type='str')
    argspec['default_role'] = dict(required=False, type='str')
    argspec['provider_config'] = dict(required=False, type='dict')
    required_one_of = [['oidc_discovery_url', 'jwks_url']]
    module = hashivault_init(argspec, supports_check_mode=True, required_one_of=required_one_of)
    result = hashivault_oidc_auth_method_config(module)
    if result.get('failed'):
        module.fail_json(**result)
    else:
        module.exit_json(**result)


@hashiwrapper
def hashivault_oidc_auth_method_config(module):
    params = module.params
    mount_point = params.get('mount_point').strip('/')
    client = hashivault_auth_client(params)
    parameters = [
        'bound_issuer',
        'jwks_ca_pem',
        'jwks_url',
        'jwt_supported_algs',
        'jwt_validation_pubkeys',
        'oidc_discovery_ca_pem',
        'oidc_discovery_url',
        'oidc_client_id',
        'oidc_client_secret',
        'default_role',
        'provider_config',
    ]
    desired_state = dict()
    for parameter in parameters:
        if params.get(parameter) is not None:
            desired_state[parameter] = params.get(parameter)
    desired_state['path'] = mount_point

    changed = False
    current_state = {}
    try:
        current_state = client.auth.oidc.read_config(path=mount_point)['data']
    except Exception:
        changed = True
    for key in desired_state.keys():
        current_value = current_state.get(key, None)
        if current_value is not None and current_value != desired_state[key]:
            changed = True
            break

    if changed and not module.check_mode:
        client.auth.oidc.configure(**desired_state)
    return {
        'changed': changed,
        "diff": {
            "before": current_state,
            "after": desired_state,
        }
    }


if __name__ == '__main__':
    main()



#!/usr/bin/env python
from ansible.module_utils.hashivault import hashivault_argspec
from ansible.module_utils.hashivault import hashivault_auth_client
from ansible.module_utils.hashivault import hashivault_init
from ansible.module_utils.hashivault import hashiwrapper

ANSIBLE_METADATA = {'status': ['stableinterface'], 'supported_by': 'community', 'version': '1.1'}
DOCUMENTATION = '''
---
module: hashivault_identity_group_alias_list
version_added: "4.7.1"
short_description: Hashicorp Vault group alias list
description:
    - Module to list group aliases from Hashicorp Vault.
options:
    mount_point:
        description:
            - mount point for identity secrets engine
        default: identity
extends_documentation_fragment: hashivault
'''
EXAMPLES = '''
---
- hosts: localhost
  tasks:
    - hashivault_identity_group_alias_list:
      register: 'vault_group_alias_list'
    - debug: msg="Group Aliases are {{vault_group_alias_list.keys}}"
'''


def main():
    argspec = hashivault_argspec()
    argspec['mount_point'] = dict(required=False, type='str', default='identity')
    argspec['method'] = dict(required=False, type='str', default='GET')
    module = hashivault_init(argspec)
    result = hashivault_identity_group_alias_list(module.params)
    if result.get('failed'):
        module.fail_json(**result)
    else:
        module.exit_json(**result)


@hashiwrapper
def hashivault_identity_group_alias_list(params):
    client = hashivault_auth_client(params)
    result = client.secrets.identity.list_group_aliases(
            mount_point=params.get('mount_point'),
            method=params.get('method').upper())
    if isinstance(result, dict):
        result = result.get('data', result)
    return {'changed': False, 'group_aliases': result}


if __name__ == '__main__':
    main()



#!/usr/bin/env python
from ansible.module_utils.hashivault import hashivault_argspec
from ansible.module_utils.hashivault import hashivault_auth_client
from ansible.module_utils.hashivault import hashivault_init
from ansible.module_utils.hashivault import hashiwrapper
from ansible.module_utils.hashivault import get_keys_updated
from hvac.exceptions import InvalidPath

ANSIBLE_METADATA = {'status': ['stableinterface'], 'supported_by': 'community', 'version': '1.1'}
DOCUMENTATION = '''
---
module: hashivault_k8s_auth_config
version_added: "4.3.0"
short_description: Hashicorp Vault k8s auth config
description:
    - Module to configure an k8s auth mount
options:
    mount_point:
        description:
            - name of the secret engine mount name.
        default: kubernetes
    kubernetes_host:
        description:
            - host must be a host string, a host:port pair, or a URL to the base of the Kubernetes API server
    token_reviewer_jwt:
        description:
            -  a service account JWT used to access the TokenReview API to validate other JWTs during login
    kubernetes_ca_cert:
        description:
            - PEM encoded CA cert for use by the TLS client used to talk with the Kubernetes API
    pem_keys:
        description:
            - Optional list of PEM-formatted public keys or certificates used to verify the signatures of Kubernetes
              service account JWTs. If a certificate is given, its public key will be extracted.
    issuer:
        description:
            - Optional JWT issuer. If no issuer is specified, then this plugin will use kubernetes.io/serviceaccount as
              the default issuer (Available in hvac 0.10.2).
extends_documentation_fragment: hashivault
'''
EXAMPLES = '''
---
- hosts: localhost
  tasks:
    - hashivault_k8s_auth_config:
        kubernetes_host: https://192.168.99.100:8443
        kubernetes_ca_cert: "-----BEGIN CERTIFICATE-----\n.....\n-----END CERTIFICATE-----"
'''


def main():
    argspec = hashivault_argspec()
    argspec['mount_point'] = dict(required=False, type='str', default='kubernetes')
    argspec['kubernetes_host'] = dict(required=False, type='str', default=None)
    argspec['token_reviewer_jwt'] = dict(required=False, type='str', default=None)
    argspec['kubernetes_ca_cert'] = dict(required=False, type='str', default=None)
    argspec['pem_keys'] = dict(required=False, type='list', default=None)
    argspec['issuer'] = dict(required=False, type='str', default=None)
    required_together = [['kubernetes_host', 'kubernetes_ca_cert']]

    module = hashivault_init(argspec, supports_check_mode=True, required_together=required_together)
    result = hashivault_k8s_auth_config(module)
    if result.get('failed'):
        module.fail_json(**result)
    else:
        module.exit_json(**result)


@hashiwrapper
def hashivault_k8s_auth_config(module):
    params = module.params
    client = hashivault_auth_client(params)
    mount_point = params.get('mount_point').strip('/')

    desired_state = dict()
    desired_state['kubernetes_host'] = params.get('kubernetes_host')
    desired_state['token_reviewer_jwt'] = params.get('token_reviewer_jwt')
    desired_state['kubernetes_ca_cert'] = params.get('kubernetes_ca_cert')
    desired_state['pem_keys'] = params.get('pem_keys')
    if params.get('issuer'):
        desired_state['issuer'] = params.get('issuer')
    desired_state['mount_point'] = mount_point

    try:
        current_state = client.auth.kubernetes.read_config(mount_point=mount_point)
    except InvalidPath:
        current_state = {}

    ignore_list = [
        'mount_point',
        'token_reviewer_jwt',
    ]
    keys_updated = get_keys_updated(desired_state, current_state, ignore_list)
    if 'pem_keys' in keys_updated:
        if current_state.get('pem_keys', []) == []:
            keys_updated.remove('pem_keys')
    if not keys_updated:
        return {'changed': False}

    if not module.check_mode:
        client.auth.kubernetes.configure(**desired_state)
    return {'changed': True, 'keys_updated': keys_updated}


if __name__ == '__main__':
    main()



#!/usr/bin/env python
from ansible.module_utils.hashivault import hashivault_argspec
from ansible.module_utils.hashivault import hashivault_client
from ansible.module_utils.hashivault import hashivault_init
from ansible.module_utils.hashivault import hashiwrapper

ANSIBLE_METADATA = {'status': ['stableinterface'], 'supported_by': 'community', 'version': '1.1'}
DOCUMENTATION = '''
---
module: hashivault_generate_root_init
version_added: "3.14.0"
short_description: Hashicorp Vault generate root token init module
description:
    - Module to start root token generation of Hashicorp Vault.
options:
    secret_shares:
        description:
            - specifies the number of shares to split the master key into.
        default: 5
    secret_threshold:
        description:
            - specifies the number of shares required to reconstruct the master key.
        default: 3
    pgp_key:
        description:
            - specifies PGP public keys used to encrypt the output root token.
        default: None
    otp:
        description:
            - must specify either pgp_key or otp
        default: None
extends_documentation_fragment: hashivault
'''
EXAMPLES = '''
---
- hosts: localhost
  tasks:
    - hashivault_generate_root_init:
        pgp_key: key
'''


def main():
    argspec = hashivault_argspec()
    argspec['pgp_key'] = dict(required=False, type='str', default=None)
    argspec['otp'] = dict(required=False, type='str', default=None)
    module = hashivault_init(argspec)
    result = hashivault_generate_root_init(module.params)
    if result.get('failed'):
        module.fail_json(**result)
    else:
        module.exit_json(**result)


@hashiwrapper
def hashivault_generate_root_init(params):
    client = hashivault_client(params)
    # Check if rekey is on-going
    status = client.sys.read_root_generation_progress()
    if status['started']:
        return {'changed': False}
    pgp_key = params.get('pgp_key')
    otp = params.get('otp')
    return {'status': client.sys.start_root_token_generation(otp=otp, pgp_key=pgp_key), 'changed': True}


if __name__ == '__main__':
    main()



#!/usr/bin/env python
from ansible.module_utils.hashivault import hashivault_argspec
from ansible.module_utils.hashivault import hashivault_auth_client
from ansible.module_utils.hashivault import hashivault_init
from ansible.module_utils.hashivault import hashiwrapper

ANSIBLE_METADATA = {'status': ['stableinterface'], 'supported_by': 'community', 'version': '1.1'}
DOCUMENTATION = '''
---
module: hashivault_consul_secret_engine_config
version_added: "4.4.7"
short_description: Hashicorp Vault consul secrets engine config
description:
    - Module to configure the consul secrets engine
options:
    mount_point:
        description:
            - name of the secret engine mount name.
        default: consul
    consul_address:
        description:
            - Specifies the address of the Consul instance, provided as "host:port" like "127.0.0.1:8500"
    scheme:
        description:
            -  Specifies the URL scheme to use
    consul_token:
        description:
            - Specifies the Consul ACL token to use. This must be a management type token.
extends_documentation_fragment: hashivault
'''
EXAMPLES = '''
---
- hosts: localhost
  tasks:
    - hashivault_consul_secret_engine_config:
        consul_address: consul.local:8500
        scheme: https
        consul_token: myAwesomeConsulManagementToken
'''


def main():
    argspec = hashivault_argspec()
    argspec['mount_point'] = dict(required=False, type='str', default='consul')
    argspec['consul_address'] = dict(required=True, type='str')
    argspec['scheme'] = dict(required=True, type='str')
    argspec['consul_token'] = dict(required=True, type='str')

    module = hashivault_init(argspec, supports_check_mode=True)
    result = hashivault_consul_secret_engine_config(module)
    if result.get('failed'):
        module.fail_json(**result)
    else:
        module.exit_json(**result)


@hashiwrapper
def hashivault_consul_secret_engine_config(module):
    params = module.params
    client = hashivault_auth_client(params)
    mount_point = params.get('mount_point').strip('/')
    consul_address = params.get('consul_address')
    scheme = params.get('scheme')
    token = params.get('consul_token')

    if (mount_point + "/") not in client.sys.list_mounted_secrets_engines()['data']:
        return {'failed': True, 'msg': 'Consul secret engine is not enabled', 'rc': 1}

    if module.check_mode:
        return {'changed': True}

    response = client.secrets.consul.configure_access(consul_address, token, scheme=scheme, mount_point=mount_point)
    if response.ok:
        return {'changed': True}
    return {'failed': True, 'msg': response.text}


if __name__ == '__main__':
    main()



#!/usr/bin/env python
from ansible.module_utils.hashivault import hashivault_argspec
from ansible.module_utils.hashivault import hashivault_auth_client
from ansible.module_utils.hashivault import hashivault_init
from ansible.module_utils.hashivault import hashiwrapper
import json

ANSIBLE_METADATA = {'status': ['stableinterface'], 'supported_by': 'community', 'version': '1.1'}
DOCUMENTATION = '''
---
module: hashivault_db_secret_engine_role
version_added: "3.17.8"
short_description: Hashicorp Vault database secret engine role
description:
    - Module to define a database role that vault can generate dynamic credentials for vault
options:
    mount_point:
        description:
            - name of the secret engine mount name.
        default: database
    name:
        description:
            - name of the role in vault
    state:
        description:
            - state of the object. choices: present, absent
        default: present
    token_ttl:
        description:
            - The TTL period of tokens issued using this role in seconds.
    token_max_ttl:
        description:
            - The maximum allowed lifetime of tokens issued in seconds using this role.
    role_file:
        description:
            - file with a json object containing play parameters. pass all params but name, state, mount_point which
              stay in the ansible play
    db_name:
        description:
            - name of the db configuration youre referencing. in my opinion, this should be called 'db connection' but
              hashi calls it db_name
    creation_statements:
        type: list
        description:
            - "Specifies the database statements executed to create and configure a user. make sure your account for
              variables like this {{'{{name}}'}}"
    revocation_statements:
        type: list
        description:
            - Specifies the database statements to be executed to revoke a user. See the plugin's API page for more
              information on support and formatting for this parameter.
    rollback_statements:
        type: list
        description:
            - Specifies the database statements to be executed rollback a create operation in the event of an error. Not
              every plugin type will support this functionality. See the plugin's API page for more information on
              support and formatting for this parameter.
    renew_statements:
        type: list
        description:
            - Specifies the database statements to be executed to renew a user. Not every plugin type will support this
              functionality. See the plugin's API page for more information on support and formatting for this
              parameter.
extends_documentation_fragment: hashivault
'''
EXAMPLES = '''
---
- hosts: localhost
  tasks:
      hashivault_db_secret_engine_role:
        name: tester
        db_name: test
        creation_statements: []


    - hashivault_db_secret_engine_role:
        name: tester
        role_file: "/Users/dmullen/git/namespaces/test-args/azure/args-db-role-file.json"
        state: "present"
'''


def main():
    argspec = hashivault_argspec()
    argspec['name'] = dict(required=True, type='str')
    argspec['state'] = dict(required=False, type='str', default='present', choices=['present', 'absent'])
    argspec['role_file'] = dict(required=False, type='str')
    argspec['mount_point'] = dict(required=False, type='str', default='database')
    argspec['token_ttl'] = dict(required=False, type='int', default=0)
    argspec['token_max_ttl'] = dict(required=False, type='int', default=0)
    argspec['creation_statements'] = dict(required=False, type='list', default=[])
    argspec['revocation_statements'] = dict(required=False, type='list', default=[])
    argspec['rollback_statements'] = dict(required=False, type='list', default=[])
    argspec['renew_statements'] = dict(required=False, type='list', default=[])
    argspec['db_name'] = dict(required=False, type='str')

    module = hashivault_init(argspec, supports_check_mode=True)
    result = hashivault_db_secret_engine_role(module)
    if result.get('failed'):
        module.fail_json(**result)
    else:
        module.exit_json(**result)


@hashiwrapper
def hashivault_db_secret_engine_role(module):
    params = module.params
    client = hashivault_auth_client(params)
    mount_point = params.get('mount_point').strip('/')
    role_file = params.get('role_file')
    name = params.get('name').strip('/')
    state = params.get('state')
    desired_state = dict()

    # if role_file is set, set desired_state to contents
    # else take values from ansible play
    if role_file:
        desired_state = json.loads(open(params.get('role_file'), 'r').read())
    else:
        desired_state['default_ttl'] = params.get('token_ttl')
        desired_state['max_ttl'] = params.get('token_max_ttl')
        desired_state['creation_statements'] = params.get('creation_statements')
        desired_state['revocation_statements'] = params.get('revocation_statements')
        desired_state['rollback_statements'] = params.get('rollback_statements')
        desired_state['db_name'] = params.get('db_name')

    # check if role exists
    exists = False
    changed = False
    try:
        client.secrets.database.read_role(name=name, mount_point=mount_point)
        exists = True
    except Exception:
        pass

    if (exists and state == 'absent') or (not exists and state == 'present'):
        changed = True

    # compare current_state to desired_state
    if exists and state == 'present' and not changed:
        current_state = client.secrets.database.read_role(name=name, mount_point=mount_point)['data']
        for k, v in desired_state.items():
            if v != current_state[k]:
                changed = True
    elif exists and state == 'absent':
        changed = True

    # make the changes!

    if changed and state == 'present' and not module.check_mode:
        client.secrets.database.create_role(name=name, mount_point=mount_point, **desired_state)

    elif changed and state == 'absent' and not module.check_mode:
        client.secrets.database.delete_role(name=name, mount_point=mount_point)

    return {'changed': changed}


if __name__ == '__main__':
    main()



#!/usr/bin/env python
ANSIBLE_METADATA = {'status': ['stableinterface'], 'supported_by': 'community', 'version': '1.1'}
DOCUMENTATION = '''
---
module: hashivault_write_from_file
version_added: "3.8.3"
short_description: Hashicorp Vault write file module
description:
    - "Writes a file encoded in base64 to Hashicorp Vault. Implementation in
     `/plugins/action/hashivault_write_from_file.py`."
options:
    secret:
        description:
            - vault secret to write.
    key:
        description:
            - secret key/name of file to write to vault.
    dest:
        description:
            - fully qualified path name of file to read from remote host.
    update:
        description:
            - Update secret rather than overwrite.
        default: True
extends_documentation_fragment: hashivault
'''
EXAMPLES = '''
---
- hosts: localhost
  tasks:
    - hashivault_write_from_file:
        secret: giant
        key: foo.dat
        path: /tmp/foo.dat
'''



#!/usr/bin/env python
from ansible.module_utils.hashivault import hashivault_auth_client
from ansible.module_utils.hashivault import hashivault_argspec
from ansible.module_utils.hashivault import hashivault_init
from ansible.module_utils.hashivault import hashiwrapper
from hvac.exceptions import InvalidPath

ANSIBLE_METADATA = {
    "status": ["preview"],
    "supported_by": "community",
    "version": "1.1",
}
DOCUMENTATION = r"""
---
module: hashivault_token_role_list
version_added: "4.7.0"
short_description: Hashicorp Vault Token List Roles
description:
    - This module returns a list of available token roles.
    - Only the role names are returned, not any values.
options:
    mount_point:
        default: token
        description:
            - location where the token auth engine is mounted. also known as path
extends_documentation_fragment:
    - hashivault
"""
EXAMPLES = r"""
---
- hosts: localhost
  tasks:
    - hashivault_token_role_list:
        mount_point: token
      register: roles_list
    - debug: msg="{{ roles_list.data }}"
"""
RETURN = r"""
---
data:
    description: list of roles, if auth token engine has no roles will return empty list
    returned: success
    type: list
"""


def main():
    argspec = hashivault_argspec()
    argspec["mount_point"] = dict(required=False, type="str", default="token")

    module = hashivault_init(argspec, supports_check_mode=True)
    result = hashivault_token_role_list(module)

    if result.get("failed"):
        module.fail_json(**result)
    else:
        module.exit_json(**result)


@hashiwrapper
def hashivault_token_role_list(module):
    params = module.params
    client = hashivault_auth_client(params)

    mount_point = params.get("mount_point").strip("/")

    try:
        return {
            "data": client.auth.token.list_roles(mount_point=mount_point)
            .get("data")
            .get("keys")
        }
    except InvalidPath as exc:
        if len(exc.errors) == 0:  # Path is valid but no roles exist.
            return {"data": []}
        return {"failed": True, "data": [], "msg": str(exc)}
    except Exception as exc:
        return {"failed": True, "data": [], "msg": str(exc)}


if __name__ == "__main__":
    main()



#!/usr/bin/env python
from ansible.module_utils.hashivault import hashivault_argspec
from ansible.module_utils.hashivault import hashivault_auth_client
from ansible.module_utils.hashivault import hashivault_init
from ansible.module_utils.hashivault import hashiwrapper

ANSIBLE_METADATA = {'status': ['stableinterface'], 'supported_by': 'community', 'version': '1.1'}
DOCUMENTATION = '''
---
module: hashivault_auth_list
version_added: "2.2.0"
short_description: Hashicorp Vault auth list module
description:
    - Module to list authentication backends in Hashicorp Vault.
extends_documentation_fragment: hashivault
'''
EXAMPLES = '''
---
- hosts: localhost
  tasks:
    - hashivault_auth_list:
      register: 'hashivault_auth_list'
'''


def main():
    argspec = hashivault_argspec()
    module = hashivault_init(argspec)
    result = hashivault_auth_list(module.params)
    if result.get('failed'):
        module.fail_json(**result)
    else:
        module.exit_json(**result)


@hashiwrapper
def hashivault_auth_list(params):
    client = hashivault_auth_client(params)
    result = client.sys.list_auth_methods()
    if isinstance(result, dict):
        result = result.get('data', result)
    return {'changed': False, 'backends': result}


if __name__ == '__main__':
    main()



#!/usr/bin/env python

import copy

import yaml

from ansible.module_utils.hashivault import hashivault_argspec
from ansible.module_utils.hashivault import hashivault_auth_client
from ansible.module_utils.hashivault import hashivault_init
from ansible.module_utils.hashivault import hashivault_normalize_from_doc
from ansible.module_utils.hashivault import hashiwrapper


ANSIBLE_METADATA = {
    "status": ["preview"],
    "supported_by": "community",
    "version": "1.1",
}
DOCUMENTATION = r"""
---
module: hashivault_ssh_role
version_added: "4.7.0"
short_description: Hashicorp Vault SSH Create/Update/Delete Role
description:
    - This module creates or updates the role definition.
    - Note that the `allowed_domains`, `allow_subdomains`, `allow_glob_domains`, and `allow_any_name` attributes are
      additive; between them nearly and across multiple roles nearly any issuing policy can be accommodated.
      `server_flag`, `client_flag`, and `code_signing_flag` are additive as well.
    - If a client requests a certificate that is not allowed by the CN policy in the role, the request is denied.
options:
    mount_point:
        default: ssh
        description:
            - location where secrets engine is mounted. also known as path
    name:
        required: true
        description:
            - Specifies the name of the role to create.
    role_file:
        description:
            - file with a json object containing play parameters. pass all params but name, state, mount_point which
              stay in the ansible play
    state:
        description:
            - Do you want for this config to be present or absent
        choices: ["present", "absent"]
        default: present
    config:
        type: dict
        description:
            - Collection of properties from ssh role
              U(https://www.vaultproject.io/api/secret/ssh#create-role)
        suboptions:
            key:
                type: str
                default: ""
                description:
                    - Specifies the name of the registered key in Vault. Before
                      creating the role, use the `keys/` endpoint to create a
                      named key. This is required for "Dynamic Key" type.
            admin_user:
                type: str
                default: ""
                description:
                    - Specifies the admin user at remote host. The shared key
                      being registered should be for this user and should have
                      root or sudo privileges. Every time a dynamic credential
                      is generated for a client, Vault uses this admin username
                      to login to remote host and install the generated
                      credential. This is required for Dynamic Key type.
            default_user:
                type: str
                default: ""
                description:
                    - Specifies the default username for which a credential
                      will be generated. When the endpoint `creds/` is used
                      without a username, this value will be used as default
                      username. Its recommended to create individual roles for
                      each username to ensure absolute isolation between
                      usernames. This is required for Dynamic Key type and OTP
                      type.
                    - For the CA type, if you wish this to be a valid
                      principal, it must also be in `allowed_users`.
            cidr_list:
                type: str
                default: ""
                description:
                    - Specifies a comma separated list of CIDR blocks for which
                      the role is applicable for. It is possible that a same
                      set of CIDR blocks are part of multiple roles. This is a
                      required parameter, unless the role is registered under
                      the `/config/zeroaddress` endpoint.
            exclude_cidr_list:
                type: str
                default: ""
                description:
                    - Specifies a comma-separated list of CIDR blocks. IP
                      addresses belonging to these blocks are not accepted by
                      the role. This is particularly useful when big CIDR
                      blocks are being used by the role and certain parts need
                      to be kept out.
            port:
                type: int
                default: 22
                description:
                    - Specifies the port number for SSH connection. Port number
                      does not play any role in OTP generation. For the `otp`
                      secrets engine type, this is just a way to inform the
                      client about the port number to use. The port number will
                      be returned to the client by Vault along with the OTP.
            key_type:
                type: str
                choices: ["otp", "dynamic", "ca"]
                required: true
                description:
                    - Specifies the type of credentials generated by this role.
                      This can be either `otp`, `dynamic` or `ca`.
            key_bits:
                type: int
                choices: [1024, 2048]
                default: 1024
                description:
                    - Specifies the length of the RSA dynamic key in bits. This
                      can be either `1024` or `2048`.
            install_script:
                type: str
                default: ""
                description:
                    - Specifies the script used to install and uninstall public
                      keys in the target machine. Defaults to the built-in
                      script.
            allowed_users:
                type: str
                default: ""
                description:
                    - |-
                      If this option is not specified, or if it is `*`, the
                      client can request a credential for any valid user at the
                      remote host, including the admin user. If only certain
                      usernames are to be allowed, then this list enforces it.
                      If this field is set, then credentials can only be
                      created for `default_user` and usernames present in this
                      list. Setting this option will enable all the users with
                      access to this role to fetch credentials for all other
                      usernames in this list. When `allowed_users_template` is
                      set to `true`, this field can contain an identity template
                      with any prefix or suffix, like
                      `ssh-{{identity.entity.id}}-user`. Use with caution. N.B.:
                      if the type is `ca`, an empty list does not allow any
                      user; instead you must use `*` to enable this behavior.
            allowed_users_template:
                type: bool
                default: false
                description:
                    - If set, `allowed_users` can be specified using identity
                      template policies. Non-templated users are also
                      permitted.
            allowed_domains:
                type: str
                default: ""
                description:
                    - The list of domains for which a client can request a host
                      certificate. If this option is explicitly set to `"*"`,
                      then credentials can be created for any domain. See also
                      `allow_bare_domains` and `allow_subdomains`.
            key_option_specs:
                type: str
                default: ""
                description:
                    - |-
                      Specifies a comma separated option specification which
                      will be prefixed to RSA keys in the remote host's
                      authorized_keys file. N.B.: Vault does not check this
                      string for validity.
            ttl:
                type: str
                default: ""
                description:
                    - Specifies the Time To Live value provided as a string
                      duration with time suffix. Hour is the largest suffix. If
                      not set, uses the system default value or the value of
                      `max_ttl`, whichever is shorter.
            max_ttl:
                type: str
                default: ""
                description:
                    - Specifies the maximum Time To Live provided as a string
                      duration with time suffix. Hour is the largest suffix. If
                      not set, defaults to the system maximum lease TTL.
            allowed_critical_options:
                type: str
                default: ""
                description:
                - Specifies a comma-separated list of critical options that
                  certificates can have when signed. To allow any critical
                  options, set this to an empty string. Will default to
                  allowing any critical options.
            allowed_extensions:
                type: str
                default: ""
                description:
                    - Specifies a comma-separated list of extensions that
                      certificates can have when signed. To allow a user to
                      specify any extension, set this to `"*"`. If not set, users
                      will not be allowed to specify extensions and will get
                      the extensions specified within `default_extensions`. For
                      the list of extensions, take a look at the sshd manual's
                      U(https://man.openbsd.org/sshd#AUTHORIZED_KEYS_FILE_FORMAT)
                      `AUTHORIZED_KEYS FILE FORMAT` section. You should add a
                      `permit-` before the name of extension to allow it.
            default_critical_options:
                type: dict
                default: null
                description:
                    - Specifies a map of critical options certificates should
                      have if none are provided when signing. This field takes
                      in key value pairs in JSON format. Note that these are
                      not restricted by `allowed_critical_options`. Defaults to
                      none.
            default_extensions:
                type: dict
                default: null
                description:
                    - Specifies a map of extensions certificates should have if
                      none are provided when signing. This field takes in key
                      value pairs in JSON format. Note that these are not
                      restricted by `allowed_extensions`. Defaults to none.
            allow_user_certificates:
                type: bool
                default: false
                description:
                    - Specifies if certificates are allowed to be signed for
                      use as a 'user'.
            allow_host_certificates:
                type: bool
                default: false
                description:
                    - Specifies if certificates are allowed to be signed for
                      use as a 'host'.
            allow_bare_domains:
                type: bool
                default: false
                description:
                    - Specifies if host certificates that are requested are
                      allowed to use the base domains listed in
                      `allowed_domains`, e.g. "example.com". This is a separate
                      option as in some cases this can be considered a security
                      threat.
            allow_subdomains:
                type: bool
                default: false
                description:
                    - Specifies if host certificates that are requested are
                      allowed to be subdomains of those listed in
                      `allowed_domains`, e.g. if "example.com" is part of
                      `allowed_domains`, this allows "foo.example.com".
            allow_user_key_ids:
                type: bool
                default: false
                description:
                    - Specifies if users can override the key ID for a signed
                      certificate with the "key_id" field. When false, the key
                      ID will always be the token display name. The key ID is
                      logged by the SSH server and can be useful for auditing.
            key_id_format:
                type: str
                default: ""
                description:
                    - |-
                      When supplied, this value specifies a custom format for
                      the key id of a signed certificate. The following
                      variables are available for use: '{{token_display_name}}'
                      - The display name of the token used to make the request.
                      '{{role_name}}' - The name of the role signing the
                      request. '{{public_key_hash}}' - A SHA256 checksum of
                      the public key that is being signed. e.g.
                      "custom-keyid-{{token_display_name}}"
            allowed_user_key_lengths:
                type: dict
                default: null
                description:
                    - Specifies a map of ssh key types and their expected sizes
                      which are allowed to be signed by the CA type.
            algorithm_signer:
                type: str
                choices: ["", "ssh-rsa", "rsa-sha2-256", "rsa-sha2-512"]
                default: ""
                description:
                    - Algorithm to sign keys with. Valid values are `ssh-rsa`,
                      `rsa-sha2-256`, and `rsa-sha2-512`. This value may be left
                      blank to use the signer's default algorithm, and must be
                      left blank for CA key types other than RSA. Note that
                      `ssh-rsa` is now considered insecure and is not supported
                      by current OpenSSH versions.
extends_documentation_fragment:
    - hashivault
"""
EXAMPLES = r"""
---
- hosts: localhost
  tasks:
    - hashivault_ssh_role:
        name: tester
        config:
            key_type: ca
            allowed_users: tester
            default_user: tester

    - hashivault_ssh_role:
        name: tester
        role_file: "/opt/vault/etc/roles/ssh-tester.json"
        state: "present"
"""


def main():
    argspec = hashivault_argspec()
    argspec["name"] = dict(required=True, type="str")
    argspec["state"] = dict(
        required=False, type="str", default="present", choices=["present", "absent"]
    )
    argspec["mount_point"] = dict(required=False, type="str", default="ssh")
    argspec["role_file"] = dict(required=False, type="str")
    argspec["config"] = dict(required=False, type="dict")

    supports_check_mode = True

    module = hashivault_init(argspec, supports_check_mode)
    result = hashivault_ssh_role(module)

    if result.get("failed"):
        module.fail_json(**result)
    else:
        module.exit_json(**result)


@hashiwrapper
def hashivault_ssh_role(module):
    params = module.params
    client = hashivault_auth_client(params)

    name = params.get("name").strip("/")
    mount_point = params.get("mount_point").strip("/")
    state = params.get("state")
    role_file = params.get("role_file")
    config = params.get("config")
    path = f"roles/{name}"

    desired_state = {}
    exists = False

    if role_file:
        with open(role_file, "rt") as fp:
            desired_state = json.safe_load(fp.read())
    elif config:
        desired_state = copy.deepcopy(config)

    changed = False

    try:
        current_state = client.secrets.pki.read_role(
            name=name, mount_point=mount_point
        ).get("data")
    except Exception:
        current_state = {}

    if current_state:
        exists = True

    if (exists and state == "absent") or (not exists and state == "present"):
        changed = True

    # compare current_state to config (desired_state before normalization)
    if exists and state == "present" and not changed:
        # Update all keys not present in the desired_state with data from the
        # current_state, to ensure a proper diff output.
        for key in current_state:
            if desired_state.get(key) is None:
                desired_state[key] = current_state[key]

        changed = desired_state != current_state

    # make the changes!
    if changed and state == "present":
        if not module.check_mode:
            # Before posting the desired state to the vault api we need to
            # normalize some keys. This is a quirk of the vault api that it
            # expects a different data format in the PUT/POST endpoint than
            # it returns in the GET endpoint.
            doc = yaml.safe_load(DOCUMENTATION)
            args = doc.get("options").get("config").get("suboptions")
            try:
                data = hashivault_normalize_from_doc(module, desired_state, args)
            except Exception as e:
                return e.args[0]
            # create or update
            client.secrets.kv.v1.create_or_update_secret(
                mount_point=mount_point,
                path=path,
                secret=data,
            )
    elif changed and state == "absent":
        if not module.check_mode:
            # delete
            client.secrets.kv.v1.delete_secret(
                mount_point=mount_point,
                path=path,
            )
        # after deleting it the item is no more
        desired_state = {}

    return {
        "changed": changed,
        "diff": {
            "before": current_state,
            "after": desired_state,
        },
    }


if __name__ == "__main__":
    main()



#!/usr/bin/env python
from ansible.module_utils.hashivault import hashivault_argspec
from ansible.module_utils.hashivault import hashivault_client
from ansible.module_utils.hashivault import hashivault_init
from ansible.module_utils.hashivault import hashiwrapper

ANSIBLE_METADATA = {'status': ['stableinterface'], 'supported_by': 'community', 'version': '1.1'}
DOCUMENTATION = '''
---
module: hashivault_init
version_added: "2.2.0"
short_description: Hashicorp Vault init enable module
description:
    - Module to init Hashicorp Vault.
options:
    seal_method:
        description:
            - specifies if `secret_*` or `recovery_*` options should be used.
        default: secret
    secret_shares:
        description:
            - specifies the number of shares to split the master key into.
        default: 5
    secret_threshold:
        description:
            - specifies the number of shares required to reconstruct the master key.
        default: 3
    pgp_keys:
        description:
            - specifies an array of PGP public keys used to encrypt the output unseal keys.
        default: []
    root_token_pgp_key:
        description:
            - specifies a PGP public key used to encrypt the initial root token.
        default: None
    stored_shares:
        description:
            - specifies the number of shares that should be encrypted.
        default: None
    recovery_shares:
        description:
            - specifies the number of shares to split the recovery key into.
        default: 5
    recovery_threshold:
        description:
            - specifies the number of shares required to reconstruct the recovery key.
        default: 3
    recovery_pgp_keys:
        description:
            - specifies an array of PGP public keys used to encrypt the output recovery keys.
        default: None
extends_documentation_fragment: hashivault
'''
EXAMPLES = '''
---
- hosts: localhost
  tasks:
    - hashivault_init:

- hosts: localhost
  tasks:
    - hashivault_init:
        secret_shares: 7
        secret_threshold: 4
'''


def main():
    argspec = hashivault_argspec()
    argspec['seal_method'] = dict(
        required=False, type='str', default='secret', choices=['secret', 'recovery']
    )
    argspec['secret_shares'] = dict(required=False, type='int', default=None)
    argspec['secret_threshold'] = dict(required=False, type='int', default=None)
    argspec['pgp_keys'] = dict(required=False, type='list', default=None)
    argspec['root_token_pgp_key'] = dict(required=False, type='str', default=None)
    argspec['stored_shares'] = dict(required=False, type='int', default=None)
    argspec['recovery_shares'] = dict(required=False, type='int', default=None)
    argspec['recovery_threshold'] = dict(required=False, type='int', default=None)
    argspec['recovery_pgp_keys'] = dict(required=False, type='list', default=None)
    module = hashivault_init(argspec)
    additional_parameter_handling(module)
    result = hashivault_initialize(module.params)
    if result.get('failed'):
        module.fail_json(**result)
    else:
        module.exit_json(**result)


def additional_parameter_handling(module):
    '''Additional parameter validation'''

    # Default shares and thresholds depending on the seal method
    if module.params.get('seal_method') == 'secret':
        module.params['secret_shares'] = (
            5
            if module.params.get('secret_shares') is None
            else module.params.get('secret_shares')
        )
        module.params['secret_threshold'] = (
            3
            if module.params.get('secret_threshold') is None
            else module.params.get('secret_threshold')
        )

        if (
            module.params.get('recovery_shares')
            or module.params.get('recovery_threshold')
            or module.params.get('recovery_pgp_keys')
        ):
            module.fail_json(
                msg="'recovery_*' options are only valid if 'seal_method' == 'recovery'",
                changed=False,
            )
    else:
        module.params['recovery_shares'] = (
            5
            if module.params.get('recovery_shares') is None
            else module.params.get('recovery_shares')
        )
        module.params['recovery_threshold'] = (
            3
            if module.params.get('recovery_threshold') is None
            else module.params.get('recovery_threshold')
        )

        if module.params.get('secret_shares') or module.params.get('secret_threshold'):
            module.fail_json(
                msg="'secret_*' options are only valid if 'seal_method' == 'secret'",
                changed=False,
            )


@hashiwrapper
def hashivault_initialize(params):
    client = hashivault_client(params)
    if client.sys.is_initialized():
        return {'changed': False}
    result = {'changed': True}
    secret_shares = params.get('secret_shares')
    secret_threshold = params.get('secret_threshold')
    pgp_keys = params.get('pgp_keys')
    root_token_pgp_key = params.get('root_token_pgp_key')
    stored_shares = params.get('stored_shares')
    recovery_shares = params.get('recovery_shares')
    recovery_threshold = params.get('recovery_threshold')
    recovery_pgp_keys = params.get('recovery_pgp_keys')
    result.update(
        client.sys.initialize(
            secret_shares=secret_shares,
            secret_threshold=secret_threshold,
            pgp_keys=pgp_keys,
            root_token_pgp_key=root_token_pgp_key,
            stored_shares=stored_shares,
            recovery_shares=recovery_shares,
            recovery_threshold=recovery_threshold,
            recovery_pgp_keys=recovery_pgp_keys,
        )
    )
    return result


if __name__ == '__main__':
    main()



#!/usr/bin/env python

import copy

import yaml

from ansible.module_utils.hashivault import hashivault_argspec
from ansible.module_utils.hashivault import hashivault_auth_client
from ansible.module_utils.hashivault import hashivault_init
from ansible.module_utils.hashivault import hashivault_normalize_from_doc
from ansible.module_utils.hashivault import hashiwrapper


ANSIBLE_METADATA = {
    "status": ["preview"],
    "supported_by": "community",
    "version": "1.1",
}
DOCUMENTATION = r"""
---
module: hashivault_token_role
version_added: "4.7.0"
short_description: Hashicorp Vault Token Create/Update/Delete Role
description:
    - This module creates or updates the token role definition.
options:
    mount_point:
        default: token
        description:
            - location where secrets engine is mounted. also known as path
    name:
        required: true
        description:
            - Specifies the name of the role to create.
    role_file:
        description:
            - file with a json object containing play parameters. pass all params but name, state, mount_point which
              stay in the ansible play
    state:
        description:
            - Do you want for this config to be present or absent
        choices: ["present", "absent"]
        default: present
    config:
        type: dict
        description:
            - Collection of properties from token role
              U(https://www.vaultproject.io/api-docs/auth/token#create-update-token-role)
        suboptions:
            allowed_policies:
                type: list
                description:
                - If set, tokens can be created with any subset of the policies
                  in this list, rather than the normal semantics of tokens being
                  a subset of the calling token's policies. The parameter is a
                  comma-delimited string of policy names. If at creation time
                  `no_default_policy` is not set and `"default"` is not
                  contained in `disallowed_policies` or glob matched in
                  `disallowed_policies_glob`, the `"default"` policy will be
                  added to the created token automatically.
            disallowed_policies:
                type: list
                description:
                - If set, successful token creation via this role will require
                  that no policies in the given list are requested. The parameter
                  is a comma-delimited string of policy names. Adding `"default"`
                  to this list will prevent `"default"` from being added
                  automatically to created tokens.
            allowed_policies_glob:
                type: list
                description:
                - If set, tokens can be created with any subset of glob matched
                  policies in this list, rather than the normal semantics of
                  tokens being a subset of the calling token's policies. The
                  parameter is a comma-delimited string of policy name globs.
                  If at creation time `no_default_policy` is not set and
                  `"default"` is not contained in `disallowed_policies` or glob
                  matched in `disallowed_policies_glob`, the `"default"` policy
                  will be added to the created token automatically. If combined
                  with `allowed_policies` policies need to only match one of the
                  two lists to be permitted. Note that unlike `allowed_policies`
                  the policies listed in `allowed_policies_glob` will not be
                  added to the token when no policies are specified in the
                  call to `/auth/token/create/:role_name`.
            disallowed_policies_glob:
                type: list
                description:
                - If set, successful token creation via this role will require
                  that no requested policies glob match any of policies in this
                  list. The parameter is a comma-delimited string of policy
                  name globs. Adding any glob that matches `"default"` to this
                  list will prevent `"default"` from being added automatically
                  to created tokens. If combined with `disallowed_policies`
                  policies need to only match one of the two lists to be blocked.
            orphan:
                type: bool
                default: false
                description:
                - If `true`, tokens created against this policy will be orphan
                  tokens (they will have no parent). As such, they will not be
                  automatically revoked by the revocation of any other token.
            renewable:
                type: bool
                default: true
                description:
                - Set to `false` to disable the ability of the token to be
                  renewed past its initial TTL. Setting the value to `true`
                  will allow the token to be renewable up to the system/mount
                  maximum TTL.
            path_suffix:
                type: str
                description:
                - If set, tokens created against this role will have the given
                  suffix as part of their path in addition to the role name.
                  This can be useful in certain scenarios, such as keeping the
                  same role name in the future but revoking all tokens created
                  against it before some point in time. The suffix can be
                  changed, allowing new callers to have the new suffix as part
                  of their path, and then tokens with the old suffix can be
                  revoked via `/sys/leases/revoke-prefix`.
            allowed_entity_aliases:
                type: list
                description:
                - String or JSON list of allowed entity aliases. If set,
                  specifies the entity aliases which are allowed to be used
                  during token generation. This field supports globbing.
                  Note that `allowed_entity_aliases` is not case sensitive.
            token_bound_cidrs:
                type: list
                description:
                - List of CIDR blocks; if set, specifies blocks of IP
                  addresses which can authenticate successfully, and ties the
                  resulting token to these blocks as well.
            token_explicit_max_ttl:
                type: str
                description:
                - If set, will encode an explicit max TTL onto the token.
                  This is a hard cap even if token_ttl and token_max_ttl would
                  otherwise allow a renewal.
            token_no_default_policy:
                type: bool
                default: false
                description:
                - If set, the `default` policy will not be set on generated
                  tokens; otherwise it will be added to the policies set in
                  `token_policies`.
            token_num_uses:
                type: int
                default: 0
                description:
                - The maximum number of times a generated token may be used
                  (within its lifetime); 0 means unlimited. If you require
                  the token to have the ability to create child tokens, you
                  will need to set this value to 0.
            token_period:
                type: str
                default: ""
                description:
                - The period, if any, to set on the token.
            token_type:
                type: str
                default: ""
                description:
                - |-
                  The type of token that should be generated. Can be `service`,
                  `batch`, or `default` to use the mount's tuned default (which
                  unless changed will be `service` tokens). For token store
                  roles, there are two additional possibilities:
                  `default-service` and `default-batch` which specify the type
                  to return unless the client requests a different type at
                  generation time.
extends_documentation_fragment:
    - hashivault
"""
EXAMPLES = r"""
---
- hosts: localhost
  tasks:
    - hashivault_token_role:
        name: tester-bot
        config:
          allowed_policies:
            - tester-bot

    - hashivault_token_role:
        name: tester-bot
        role_file: "/opt/vault/etc/roles/token-tester-bot.json"
        state: "present"
"""


def main():
    argspec = hashivault_argspec()
    argspec["name"] = dict(required=True, type="str")
    argspec["state"] = dict(
        required=False, type="str", default="present", choices=["present", "absent"]
    )
    argspec["mount_point"] = dict(required=False, type="str", default="token")
    argspec["role_file"] = dict(required=False, type="str")
    argspec["config"] = dict(required=False, type="dict")

    supports_check_mode = True

    module = hashivault_init(argspec, supports_check_mode)
    result = hashivault_token_role(module)

    if result.get("failed"):
        module.fail_json(**result)
    else:
        module.exit_json(**result)


@hashiwrapper
def hashivault_token_role(module):
    params = module.params
    client = hashivault_auth_client(params)

    name = params.get("name").strip("/")
    mount_point = params.get("mount_point").strip("/")
    state = params.get("state")
    role_file = params.get("role_file")
    config = params.get("config")

    desired_state = {}
    exists = False

    if role_file:
        with open(role_file, "rt") as fp:
            desired_state = json.safe_load(fp.read())
    elif config:
        desired_state = copy.deepcopy(config)

    changed = False

    try:
        current_state = client.auth.token.read_role(
            role_name=name, mount_point=mount_point
        ).get("data")
    except Exception:
        current_state = {}

    if current_state:
        exists = True

    if (exists and state == "absent") or (not exists and state == "present"):
        changed = True

    # compare current_state to config (desired_state before normalization)
    if exists and state == "present" and not changed:
        # Update all keys not present in the desired_state with data from the
        # current_state, to ensure a proper diff output.
        for key in current_state:
            if desired_state.get(key) is None:
                desired_state[key] = current_state[key]

        changed = desired_state != current_state

    # make the changes!
    if changed and state == "present":
        if not module.check_mode:
            # Before posting the desired state to the vault api we need to
            # normalize some keys. This is a quirk of the vault api that it
            # expects a different data format in the PUT/POST endpoint than
            # it returns in the GET endpoint.
            extra_params = {}

            doc = yaml.safe_load(DOCUMENTATION)
            args = doc.get("options").get("config").get("suboptions")
            try:
                extra_params = hashivault_normalize_from_doc(module, desired_state, args)
            except Exception as e:
                return e.args[0]
            # create or update
            api_path = f"/v1/auth/{mount_point}/roles/{name}"
            client.auth.token._adapter.post(url=api_path, json=extra_params)
    elif changed and state == "absent":
        if not module.check_mode:
            # delete
            client.auth.token.delete_role(
                role_name=name,
                mount_point=mount_point,
            )
        # after deleting it the item is no more
        desired_state = {}

    return {
        "changed": changed,
        "diff": {
            "before": current_state,
            "after": desired_state,
        },
    }


if __name__ == "__main__":
    main()



#!/usr/bin/env python
from ansible.module_utils.hashivault import hashivault_auth_client
from ansible.module_utils.hashivault import hashivault_argspec
from ansible.module_utils.hashivault import hashivault_init
from ansible.module_utils.hashivault import hashiwrapper

ANSIBLE_METADATA = {'status': ['preview'], 'supported_by': 'community', 'version': '1.1'}
DOCUMENTATION = r'''
---
module: hashivault_pki_role_list
version_added: "4.5.0"
short_description: Hashicorp Vault PKI List Roles
description:
    - This module returns a list of available roles.
    - Only the role names are returned, not any values.
options:
    mount_point:
        default: pki
        description:
            - location where secrets engine is mounted. also known as path
extends_documentation_fragment:
    - hashivault
'''
EXAMPLES = r'''
---
- hosts: localhost
  tasks:
    - hashivault_pki_role_list:
        mount_point: pki
      register: roles_list
    - debug: msg="{{ roles_list.data }}"
'''
RETURN = r'''
---
data:
    description: list of roles, if pki engine has no roles will return empty list
    returned: success
    type: list
'''


def main():
    argspec = hashivault_argspec()
    argspec['mount_point'] = dict(required=False, type='str', default='pki')

    module = hashivault_init(argspec)
    result = hashivault_pki_role_list(module)

    if result.get('failed'):
        module.fail_json(**result)
    else:
        module.exit_json(**result)


@hashiwrapper
def hashivault_pki_role_list(module):
    params = module.params
    client = hashivault_auth_client(params)

    mount_point = params.get('mount_point').strip('/')

    try:
        return {'data': client.secrets.pki.list_roles(mount_point=mount_point).get('data').get('keys')}
    except Exception:
        return {'data': []}


if __name__ == '__main__':
    main()
